package cn.wisenergy.chnmuseum.party.service;

import com.baomidou.mybatisplus.mapper.Wrapper;
import cn.wisenergy.chnmuseum.party.common.util.DateUtil80;
import cn.wisenergy.chnmuseum.party.model.HttpResult;
import cn.wisenergy.chnmuseum.party.model.PageViewRecord;
import cn.wisenergy.chnmuseum.party.model.PageViews;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.client.RestTemplate;
import com.baomidou.mybatisplus.mapper.QueryWrapper;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@Service
public class PublicService {

    @Resource
    private RestTemplate restTemplate;

    @Value("${GET_ACCESSTOKEN}")
    private String GET_ACCESSTOKEN;

    @Value("${CLIENT_ID}")
    private String CLIENT_ID;

    @Value("${CLIENT_SECRET}")
    private String CLIENT_SECRET;

    @Resource
    private StringRedisTemplate hashRedisTemplate;

    @Resource
    private RedisTemplate redisTemplate;

    @Resource
    private IPageViewRecordService pageViewRecordService;

    @Resource
    private IPageViewsService pageViewsService;

    /**
     * 获取access_token
     */
    public HttpResult getAccessToken() {
        StringBuilder url = new StringBuilder(GET_ACCESSTOKEN);
        url.append("OAuth/token");
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("grant_type", "client_credentials");
        requestEntity.add("client_id", CLIENT_ID);
        requestEntity.add("client_secret", CLIENT_SECRET);
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        System.out.println(url.toString());
        return restTemplate.postForObject(url.toString(), requestEntity, HttpResult.class);
    }

    /**
     *刷新access_token，暂时注释，目前不需要用到
     *//*
    public HttpResult refreshAccessToken(String refreshToken){
        StringBuilder url = new StringBuilder(GET_ACCESSTOKEN);
        url.append("OAuth/token");
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("grant_type","refresh_token");
        requestEntity.add("refresh_token", refreshToken);
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        System.out.println(url.toString());
        return restTemplate.postForObject(url.toString(), requestEntity, HttpResult.class);
    }*/

    /**
     * 发送消息对接招呼
     */
    @PostMapping(value = "/sendMessage")
    public void sendMessage(String zhCnt, String recUsrNbr) {
        String key = "CMB_ACCESS_TOKEN";
        HttpResult token;
        //调用接口凭证
        String access_token = "";
        if (hashRedisTemplate.hasKey(key)) {
            access_token = hashRedisTemplate.opsForValue().get(key);
        } else {
            token = this.getAccessToken();
            access_token = token.getAccess_token();
            String expires_in = token.getExpires_in();
            Long expire = Long.valueOf(expires_in) - 360L;
            hashRedisTemplate.opsForValue().set(key, access_token);
            hashRedisTemplate.expire(key, expire, TimeUnit.SECONDS);
        }

        //发送人CIP号，8位，如01014760
        String sedUsrNbr = "01146666";
        //发送人姓名
        String sedUsrNam = "于思倢";

        //不必传	设定发送时间（14位数字）
//        Date sedExpTim = new Date(System.currentTimeMillis());

        //根据URL传递内容
        StringBuilder url = new StringBuilder(GET_ACCESSTOKEN);
        url.append("api/Servers/SendZhMessageText?access_token=");
        url.append(access_token);
        MultiValueMap<String, String> requestEntity = new LinkedMultiValueMap<>();
        requestEntity.add("recUsrNbr", recUsrNbr);
        requestEntity.add("sedUsrNbr", sedUsrNbr);
        requestEntity.add("sedUsrNam", sedUsrNam);
        requestEntity.add("zhCnt", zhCnt);
//        requestEntity.add("sedExpTim", sedExpTim);
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        System.out.println(url.toString());
        restTemplate.postForObject(url.toString(), requestEntity, HttpResult.class);
    }

    /**
     * 缓存页面访问量数据，超过5分钟或者数据量超过10条时直接插入数据库
     *
     * @param detailId 详情信息ID(具体的业务，网点，活动，产品ID)
     * @param userId   用户ID
     * @param bankId   网点ID
     * @param type     页面类型:1-业务办理助手2-网点服务地图3-热门产品4-热门活动
     */
    public void cacheData(String detailId, String userId, String bankId, Integer type) {
        String recordsKey = "pageViewRecords";
        String timeKey = "pageViewRecordsTime";

        Date currDateTime = DateUtil80.getCurrDateTime();
        Date currDate = DateUtil80.getCurrDate();

        PageViewRecord pageViewRecord = new PageViewRecord();
        pageViewRecord.setUserId(userId);
        pageViewRecord.setBankId(bankId);
        pageViewRecord.setCreateTime(currDateTime);//年月日时分秒
        pageViewRecord.setType(type);
        pageViewRecord.setDetailId(detailId);
        redisTemplate.opsForSet().add(recordsKey, pageViewRecord);
        if (!redisTemplate.hasKey(timeKey)) {
            //没有时间戳标记说明是第一次进入
            redisTemplate.opsForValue().set(timeKey, currDateTime);
        } else {
            //取出记录集合
            Set<PageViewRecord> recordValues = redisTemplate.opsForSet().members(recordsKey);

            int size = recordValues.size();
            //取出时间判断和当前时间有没有超过5分钟
            Date timeValue = (Date) redisTemplate.opsForValue().get(timeKey);
            long disSecond = (currDateTime.getTime() - timeValue.getTime()) / 1000;

            //缓存的数据量达到10条以上或者时间超过5分钟，进行数据存储
            if (size >= 10 || disSecond >= 300) {
                int typeOne = 0;
                int typeTwo = 0;
                int typeThree = 0;
                int typeFour = 0;
                //数据量达到10条以上，将数据插入数据库，并且清空redis
                Iterator<PageViewRecord> iterator = recordValues.iterator();
                while (iterator.hasNext()) {
                    //单条缓存的数据
                    PageViewRecord cacheValue = iterator.next();
                    pageViewRecord = new PageViewRecord();

                    pageViewRecord.setUserId(cacheValue.getUserId());
                    pageViewRecord.setBankId(cacheValue.getBankId());
                    pageViewRecord.setCreateTime(cacheValue.getCreateTime());
                    pageViewRecord.setType(cacheValue.getType());
                    pageViewRecord.setDetailId(cacheValue.getDetailId());
                    //将数据插入数据库
                    this.pageViewRecordService.insert(pageViewRecord);
                    //记录每个页面的访问数量
                    if (cacheValue.getType() == 1) {
                        typeOne += 1;
                    } else if (cacheValue.getType() == 2) {
                        typeTwo += 1;
                    } else if (cacheValue.getType() == 3) {
                        typeThree += 1;
                    } else if (cacheValue.getType() == 4) {
                        typeFour += 1;
                    }
                }

                Wrapper<PageViews> viewsWrapper = new QueryWrapper<>();
                PageViews pageViews = new PageViews();
                if (typeOne != 0) {
                    viewsWrapper.eq("type", 1);
                    PageViews one = this.pageViewsService.selectOne(viewsWrapper);
                    if (one != null) {
                        pageViews.setId(one.getId());
                        pageViews.setTimes(one.getTimes() + typeOne);
                        this.pageViewsService.updateById(pageViews);
                    } else {
                        pageViews.setType(1);
                        pageViews.setPageName("业务办理助手");
                        pageViews.setTimes(typeOne);
                        pageViews.setViewDate(currDate);
                        this.pageViewsService.insert(pageViews);
                    }
                }
                if (typeTwo != 0) {
                    viewsWrapper = new QueryWrapper<>();
                    pageViews = new PageViews();
                    viewsWrapper.eq("view_date", currDate);
                    viewsWrapper.eq("type", 2);
                    PageViews one = this.pageViewsService.selectOne(viewsWrapper);
                    if (one != null) {
                        pageViews.setId(one.getId());
                        pageViews.setTimes(one.getTimes() + typeTwo);
                        this.pageViewsService.updateById(pageViews);
                    } else {
                        pageViews.setType(2);
                        pageViews.setPageName("网点服务地图");
                        pageViews.setTimes(typeTwo);
                        pageViews.setViewDate(currDate);
                        this.pageViewsService.insert(pageViews);
                    }
                }
                if (typeThree != 0) {
                    viewsWrapper = new QueryWrapper<>();
                    pageViews = new PageViews();
                    viewsWrapper.eq("view_date", currDate);
                    viewsWrapper.eq("type", 3);
                    PageViews one = this.pageViewsService.selectOne(viewsWrapper);
                    if (one != null) {
                        pageViews.setId(one.getId());
                        pageViews.setTimes(one.getTimes() + typeThree);
                        this.pageViewsService.updateById(pageViews);
                    } else {
                        pageViews.setType(3);
                        pageViews.setPageName("热门产品");
                        pageViews.setTimes(typeThree);
                        pageViews.setViewDate(currDate);
                        this.pageViewsService.insert(pageViews);
                    }
                }
                if (typeFour != 0) {
                    viewsWrapper = new QueryWrapper<>();
                    pageViews = new PageViews();
                    viewsWrapper.eq("view_date", currDate);
                    viewsWrapper.eq("type", 4);
                    PageViews one = this.pageViewsService.selectOne(viewsWrapper);
                    if (one != null) {
                        pageViews.setId(one.getId());
                        pageViews.setTimes(one.getTimes() + typeFour);
                        this.pageViewsService.updateById(pageViews);
                    } else {
                        pageViews.setType(4);
                        pageViews.setPageName("热门活动");
                        pageViews.setTimes(typeFour);
                        pageViews.setViewDate(currDate);
                        this.pageViewsService.insert(pageViews);
                    }
                }
                //插入数据库后清空redis
                redisTemplate.opsForValue().set(timeKey, currDateTime);
                redisTemplate.delete(recordsKey);
            }
        }
    }
}
