package cn.wisenergy.chnmuseum.party.web.controller;

import cn.wisenergy.chnmuseum.party.core.annotations.OperationLog;
import cn.wisenergy.chnmuseum.party.model.BusinessInfo;
import cn.wisenergy.chnmuseum.party.service.PublicService;
import cn.wisenergy.chnmuseum.party.service.impl.BusinessInfoServiceImpl;
import cn.wisenergy.chnmuseum.party.service.impl.EmployeeServiceImpl;
import com.baomidou.mybatisplus.mapper.QueryWrapper;
import com.baomidou.mybatisplus.mapper.Wrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import cn.wisenergy.chnmuseum.party.web.controller.base.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 业务信息 前端控制器
 * </p>
 *
 * @author 杨智平
 * @since 2018-08-29
 */
@Api(tags = "业务办理指南")
@RestController
@RequestMapping("/businessInfo")
public class BusinessInfoController extends BaseController {

    private static final Logger logger = LoggerFactory.getLogger(BusinessInfoController.class);

    @Resource
    private BusinessInfoServiceImpl businessInfoService;

    @Resource
    private EmployeeServiceImpl employeeService;

    @Resource
    private PublicService publicService;

    /**
     * 获取业务服务指南
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "通过业务ID获取业务服务指南")
    @GetMapping(value = "/getById")
    @RequiresPermissions("/businessInfo/getById")
    public ResponseEntity<BusinessInfo> getById(String id) {
        try {
            BusinessInfo one = this.businessInfoService.selectOneById(id);
            String creatorName = this.employeeService.selectById(one.getCreator()).getName();
            one.setCreateEmployee(creatorName);

            if (one.getModifier() != null) {
                String modifierName = this.employeeService.selectById(one.getModifier()).getName();
                one.setModifyEmployee(modifierName);
            }
            if (null != one) {
                return ResponseEntity.ok(one);
            }
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(null);
        } catch (Exception e) {
            logger.error("查询业务信息出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * 获取业务服务指南列表
     *
     * @param name
     * @return
     */
    @ApiOperation(value = "获取业务服务指南列表信息")
    @GetMapping(value = "/getList")
    @RequiresPermissions("/businessInfo/getList")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "name", value = "业务名称", required = false, dataType = "String"),
            @ApiImplicitParam(name = "bankId", value = "所属网点Id", required = false, dataType = "String")})
    public ResponseEntity<Page<BusinessInfo>> getList(String name, String bankId, String roleId,
                                                      String currentBankId, String userId) {
        try {
            if (!"3".equals(roleId)) {
                roleId = "0";
            }
            name = StringUtils.trimToNull(name);
            Page<BusinessInfo> page = this.getPage();
            Page<BusinessInfo> bankBranchInfoPage = this.businessInfoService.selectListByName(page, name, bankId, roleId, currentBankId, userId);
            return ResponseEntity.ok(bankBranchInfoPage);
        } catch (Exception e) {
            logger.error("查询业务信息列表出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * 保存或提交业务服务指南
     */
    @OperationLog("保存或提交业务服务指南")
    @ApiOperation("保存或提交业务服务指南")
    @PostMapping(value = "/add")
    @RequiresPermissions("/businessInfo/add")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "action", value = "1-保存2-提交", required = true)})
    public ResponseEntity<Map<String, Object>> add(BusinessInfo businessInfo, Integer action, String currentBankId) {
        Map<String, Object> resultMap = new LinkedHashMap<String, Object>();
        try {
            boolean ret = false;
            Wrapper<BusinessInfo> ew = new QueryWrapper<>();
            if (StringUtils.isNoneBlank(businessInfo.getName())) {
                ew.ne("status",5);
                ew.eq("name", businessInfo.getName());
                if (businessInfo.getBankBranchId() != null && !"0".equals(businessInfo.getBankBranchId())) {
                    ew.andNew();
                    ew.eq("bank_branch_id", "0");
                    ew.or();
                    ew.eq("bank_branch_id", currentBankId);
                }
                BusinessInfo one = this.businessInfoService.selectOne(ew);
                if (one != null) {
                    resultMap.put("status", 400);
                    resultMap.put("message", "该业务名称已存在");
                    return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
                } else {
                    businessInfo.setName(StringUtils.trimToNull(businessInfo.getName()));
                }
            }
            if (StringUtils.isBlank(businessInfo.getBankBranchId())) {
                resultMap.put("status", 400);
                resultMap.put("message", "请选择所属网点");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
            }
            if (businessInfo.getSortorder() == null || "".equals(businessInfo.getSortorder())) {
                ew = new QueryWrapper<>();
                ew.setSqlSelect("max(sortorder) as sortorder");
                BusinessInfo one = this.businessInfoService.selectOne(ew);
                if (one != null) {
                    businessInfo.setSortorder(one.getSortorder() + 1);
                } else {
                    businessInfo.setSortorder(1);
                }
            }
            if (action.equals(1)) {
                //保存是到编辑中
                businessInfo.setStatus(0);
            } else if (action.equals(2)) {
                //提交是到上线审核
                businessInfo.setStatus(2);
            }
            businessInfo.setCreator(this.getUserId());
            businessInfo.setSkipLink(StringUtils.trimToNull(businessInfo.getSkipLink()));
            businessInfo.setCreateTime(new Date(System.currentTimeMillis()));
            businessInfo.setUpdateTime(businessInfo.getCreateTime());
            ret = this.businessInfoService.insert(businessInfo);

            if (!ret) {
                resultMap.put("status", 400);
                resultMap.put("message", "添加失败");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
            }
            // 201
            resultMap.put("status", 201);
            resultMap.put("message", "添加成功");
            return ResponseEntity.status(HttpStatus.CREATED).body(resultMap);
        } catch (Exception e) {
            logger.error("添加错误!", e);
        }
        resultMap.put("status", 500);
        resultMap.put("message", "操作失败");
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
    }

    @OperationLog("修改业务服务指南")
    @ApiOperation(value = "修改业务服务指南")
    @PutMapping(value = "/edit")
    @RequiresPermissions("/businessInfo/edit")
    public ResponseEntity<Map<String, Object>> edit(BusinessInfo businessInfo, Integer action, String currentBankId) {
        Map<String, Object> resultMap = new LinkedHashMap<String, Object>();
        try {
            Wrapper<BusinessInfo> ew = new QueryWrapper<>();
            if (StringUtils.isNoneBlank(businessInfo.getName())) {
                ew.ne("id", businessInfo.getId());
                ew.ne("status",5);
                ew.eq("name", businessInfo.getName().trim());
                if (businessInfo.getBankBranchId() != null && !"0".equals(businessInfo.getBankBranchId())) {
                    ew.andNew();
                    ew.eq("bank_branch_id", "0");
                    ew.or();
                    ew.eq("bank_branch_id", currentBankId);
                }
                businessInfo.setName(businessInfo.getName().trim());
            }
            BusinessInfo one = this.businessInfoService.selectOne(ew);
            if (one != null) {
                resultMap.put("status", 400);
                resultMap.put("message", "该业务已存在");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
            }
            if (action != null) {
                if (action.equals(1)) {
                    //保存是到编辑中
                    businessInfo.setStatus(0);
                } else if (action.equals(2)) {
                    //提交是到上线审核
                    businessInfo.setStatus(2);
                }
            }
            businessInfo.setModifier(this.getUserId());
            businessInfo.setUpdateTime(new Date(System.currentTimeMillis()));
            boolean ret = this.businessInfoService.updateById(businessInfo);
            if (!ret) {
                resultMap.put("status", 500);
                resultMap.put("message", "修改失败");
                return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
            } else {
                resultMap.put("status", 201);
                resultMap.put("message", "修改成功");
                return ResponseEntity.ok(resultMap);
            }
        } catch (Exception e) {
            logger.error("修改错误!", e);
        }
        resultMap.put("status", 500);
        resultMap.put("message", "修改失败");
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
    }

    /**
     * 刪除
     *
     * @param id
     * @return
     */
    @OperationLog("刪除业务服务指南")
    @ApiOperation(value = "刪除业务服务指南")
    @DeleteMapping(value = "/delete")
    @RequiresPermissions("/businessInfo/delete")
    public ResponseEntity<Map<String, Object>> delete(@RequestParam(value = "id", required = true) String id) {
        Map<String, Object> resultMap = new LinkedHashMap<String, Object>();
        try {
            Boolean ret = this.businessInfoService.deleteById(id);
            if (!ret) {
                // 删除失败, 500
                resultMap.put("status", 500);
                resultMap.put("message", "服务器错误！");
                return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
            }
            // 201
            resultMap.put("status", 201);
            resultMap.put("message", "删除成功！");
            return ResponseEntity.status(HttpStatus.CREATED).body(resultMap);
        } catch (Exception e) {
            logger.error("服务器错误!", e);
        }
        // 500
        resultMap.put("status", 500);
        resultMap.put("message", "服务器错误！");
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
    }

    /**
     * 申请上线
     *
     * @param id
     * @return
     */
    @OperationLog("申请上线单个业务服务指南")
    @ApiOperation(value = "上线审核单个业务服务指南")
    @PutMapping(value = "/enable")
    @RequiresPermissions("/businessInfo/enable")
    public ResponseEntity<Map<String, Object>> enable(String id) {
        try {
            Map<String, Object> map = new HashMap<>();
            BusinessInfo entity = new BusinessInfo();
            entity.setId(id);
            entity.setStatus(2);
            entity.setModifier(this.getUserId());
            entity.setUpdateTime(new Date(System.currentTimeMillis()));
            boolean ret = this.businessInfoService.updateById(entity);
            if (!ret) {
                map.put("status", "500");
                map.put("message", "服务器错误");
                return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(map);
            }
            map.put("status", "201");
            map.put("message", "上线成功");
            return ResponseEntity.status(HttpStatus.CREATED).body(map);
        } catch (Exception e) {
            logger.error("上线出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(
                null);
    }

    /**
     * 申请下线
     *
     * @param id
     * @return
     */
    @OperationLog("申请下线单个业务服务指南")
    @ApiOperation(value = "下线审核单个业务服务指南")
    @PutMapping(value = "/disable")
    @RequiresPermissions("/businessInfo/disable")
    public ResponseEntity<Map<String, Object>> disable(String id) {
        try {
            Map<String, Object> map = new HashMap<>();
            BusinessInfo entity = new BusinessInfo();
            entity.setId(id);
            entity.setStatus(3);
            entity.setModifier(this.getUserId());
            entity.setUpdateTime(new Date(System.currentTimeMillis()));
            boolean ret = this.businessInfoService.updateById(entity);
            if (!ret) {
                map.put("status", "400");
                map.put("message", "下线错误");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            }
            map.put("status", "201");
            map.put("message", "下线成功");
            return ResponseEntity.status(HttpStatus.CREATED).body(map);
        } catch (Exception e) {
            logger.error("下线出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(
                null);
    }

    /**
     * APP端查看业务服务指南列表
     *
     * @return
     */
    @ApiOperation(value = "APP端查看业务服务指南列表")
    @GetMapping(value = "/getAppBusinessList")
    public ResponseEntity<Page<BusinessInfo>> getAppBusinessList(String bankId, String roleId) {
        try {
            Page<BusinessInfo> page = this.getPage();
            Wrapper<BusinessInfo> ew = new QueryWrapper();
            ew.setSqlSelect("id", "name", "icon");
            if (roleId != null) {
                if (bankId != null && "3".equals(roleId)) {
                    ew.eq("bank_branch_id", bankId);
                } else if ("1".equals(roleId)) {
                    ew.eq("bank_branch_id", "0");
                }
            } else {
                ew.eq("bank_branch_id", '0');
                ew.or();
                ew.eq("bank_branch_id", bankId);
                ew.orderBy("bank_branch_id", false);
            }
            ew.andNew("status = 3 OR status = 4 OR status = 6");
            ew.orderBy("sortorder", true);
            ew.orderBy("update_time", false);
            Page<BusinessInfo> businessInfoPage = this.businessInfoService.selectPage(page, ew);
            return ResponseEntity.ok(businessInfoPage);
        } catch (Exception e) {
            logger.error("查询业务指南列表出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * APP端查看单个业务服务指南
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "APP端查看单个业务服务指南")
    @GetMapping(value = "/getAppBusinessInfoById")
    public ResponseEntity<BusinessInfo> getAppBusinessInfoById(String id, String userId, String bankId) {
        try {
            Wrapper<BusinessInfo> ew = new QueryWrapper();
            ew.setSqlSelect("name, PROCESS,guide,is_skip AS isSkip,skip_link AS skipLink");
            ew.eq("id", id);
            BusinessInfo one = this.businessInfoService.selectOne(ew);
            if (null != one) {
                //将数据放入缓存
                this.publicService.cacheData(id, userId, bankId, 1);
                return ResponseEntity.ok(one);
            }
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(null);
        } catch (Exception e) {
            logger.error("查询业务指南出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * 业务指南审核通过
     *
     * @param id
     * @return
     */
    @OperationLog("业务服务指南审核通过")
    @ApiOperation(value = "业务服务指南审核通过")
    @PutMapping(value = "/approved")
    @RequiresPermissions("/businessInfo/approved")
    public ResponseEntity<Map<String, Object>> approved(String id, Integer status) {
        try {
            Map<String, Object> map = new HashMap<>();
            boolean ret = false;
            if (StringUtils.isNotBlank(id)) {
                BusinessInfo entity = new BusinessInfo();
                entity.setId(id);
                if (status != null) {
                    if (status == 2) {
                        entity.setStatus(4);
                    } else if (status == 3) {
                        entity.setStatus(5);
                    }
                }
                entity.setReviewer(this.getUserId());
                entity.setUpdateTime(new Date(System.currentTimeMillis()));
                ret = this.businessInfoService.updateById(entity);
            } else {
                map.put("status", "400");
                map.put("message", "请选择需要审核的业务");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            }
            if (!ret) {
                map.put("status", "400");
                map.put("message", "审核通过失败");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            } else {
                map.put("status", "201");
                map.put("message", "审核通过成功");
                return ResponseEntity.status(HttpStatus.CREATED).body(map);
            }
        } catch (Exception e) {
            logger.error("审核通过失败!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
    }

    /**
     * 业务指南审核驳回
     *
     * @param id
     * @param rejectReason 驳回原因
     * @return
     */
    @OperationLog("业务服务指南审核驳回")
    @ApiOperation(value = "业务服务指南审核驳回")
    @PutMapping(value = "/reject")
    @RequiresPermissions("/businessInfo/reject")
    public ResponseEntity<Map<String, Object>> reject(String id, String rejectReason, Integer status) {
        try {
            Map<String, Object> map = new HashMap<>();
            if (rejectReason == null || "".equals(rejectReason.trim())) {
                map.put("status", "400");
                map.put("message", "请输入驳回原因！");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            }
            boolean ret = false;
            if (StringUtils.isNotBlank(id)) {
                BusinessInfo entity = new BusinessInfo();
                entity.setId(id);
                if (status != null) {
                    if (status == 2) {
                        entity.setStatus(1);
                    } else if (status == 3) {
                        entity.setStatus(6);
                    }
                }
                entity.setRejectReason(rejectReason);
                entity.setReviewer(this.getUserId());
                entity.setUpdateTime(new Date(System.currentTimeMillis()));
                ret = this.businessInfoService.updateById(entity);
            } else {
                map.put("status", "400");
                map.put("message", "请选择需要审核的业务");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            }
            if (!ret) {
                map.put("status", "400");
                map.put("message", "审核驳回失败");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            } else {
                map.put("status", "201");
                map.put("message", "审核驳回成功");
                return ResponseEntity.status(HttpStatus.CREATED).body(map);
            }
        } catch (Exception e) {
            logger.error("审核驳回失败!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
    }

    /**
     * 业务指南排序（供APP端展示内容使用）
     *
     * @param businessInfo
     * @return
     */
    @OperationLog("业务指南排序")
    @ApiOperation(value = "业务指南排序")
    @PutMapping(value = "/sort")
    @RequiresPermissions("/businessInfo/sort")
    public ResponseEntity<Map<String, Object>> sort(@RequestBody BusinessInfo businessInfo) {
        try {
            Map<String, Object> map = new HashMap<>();
            // 对顺序号赋值封装
            List<BusinessInfo> newList = new ArrayList<BusinessInfo>();
            boolean ret = false;
            if (businessInfo != null && businessInfo.getOldList().size() > 0) {
                List<BusinessInfo> oldList = businessInfo.getOldList();
                //先清空之前的排序
                String bankBranchId = businessInfo.getBankBranchId();
                Wrapper<BusinessInfo> ew = new QueryWrapper<>();
                ew.eq("bank_branch_id", bankBranchId);
                ret = this.businessInfoService.updateForSet("sortorder = null ", ew);

                //重新排序
                for (int i = 0; i < oldList.size(); i++) {
                    BusinessInfo entity = oldList.get(i);
                    entity.setSortorder(i + 1);
                    newList.add(entity);
                }
                ret = this.businessInfoService.updateBatchById(newList);
            } else {
                map.put("status", "400");
                map.put("message", "请选择需要排序的业务");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            }
            if (!ret) {
                map.put("status", "400");
                map.put("message", "排序失败");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(map);
            } else {
                map.put("status", "201");
                map.put("message", "排序成功");
                return ResponseEntity.status(HttpStatus.CREATED).body(map);
            }
        } catch (Exception e) {
            logger.error("排序失败!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build();
    }

}

