package cn.wisenergy.chnmuseum.party.web.controller;


import cn.wisenergy.chnmuseum.party.common.util.DateUtil;
import cn.wisenergy.chnmuseum.party.model.BarChart;
import cn.wisenergy.chnmuseum.party.model.BarChartData;
import cn.wisenergy.chnmuseum.party.model.PageViews;
import cn.wisenergy.chnmuseum.party.service.impl.PageViewsServiceImpl;
import com.baomidou.mybatisplus.mapper.QueryWrapper;
import com.baomidou.mybatisplus.mapper.Wrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 页面访问量
 * </p>
 *
 * @author Yang Jianlong
 * @since 2018-12-27
 */
@Api(tags = "页面访问量")
@RestController
@RequestMapping("/pageViews")
public class PageViewsController {

    @Resource
    private PageViewsServiceImpl pageViewsService;

    @ApiOperation(value = "统计页面访问量", notes = "统计页面访问量")
    @RequestMapping(value = "/statisticsPageViews", method = RequestMethod.GET)
    public BarChart statisticsPageViews(String startDate, String endDate, String statisticsType) {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        if (statisticsType == null || "".equals(statisticsType)) {
            statisticsType = "year";
        }

        if (startDate == null || "".equals(startDate) || endDate == null || "".equals(endDate)) {
            Date nowDate = new Date();
            endDate = sdf.format(nowDate);
            if ("year".equals(statisticsType)) {
                startDate = sdf.format(DateUtil.getSomeDay(nowDate, -2, statisticsType));
            } else if ("month".equals(statisticsType)) {
                startDate = sdf.format(DateUtil.getSomeDay(nowDate, -11, statisticsType));
            } else if ("day".equals(statisticsType)) {
                startDate = sdf.format(DateUtil.getSomeDay(nowDate, -29, statisticsType));
            }
        }

        if ("year".equals(statisticsType)) {
            sdf = new SimpleDateFormat("yyyy");
        } else if ("month".equals(statisticsType)) {
            sdf = new SimpleDateFormat("yyyy-MM");
        }

        BarChart barChart = new BarChart();

        Map<String, List<Object>> legend = new HashMap<>();
        List<Object> legendData = new ArrayList<>();
        legendData.add("业务办理助手");
        legendData.add("网点服务地图");
        legendData.add("热门产品");
        legendData.add("热门活动");
        legend.put("data", legendData);
        barChart.setLegend(legend);

        Wrapper<PageViews> viewsWrapper = new QueryWrapper<>();
        if ("year".equals(statisticsType)) {
            viewsWrapper.setSqlSelect("type,(CASE type WHEN 1 THEN '业务办理助手' WHEN 2 THEN '网点服务地图' WHEN 3 THEN '热门产品' else '热门活动' END) AS typeName," +
                    "DATE_FORMAT(view_date, '%Y') AS dateType,SUM(times) AS totalTimes");
        } else if ("month".equals(statisticsType)) {
            viewsWrapper.setSqlSelect("type,(CASE type WHEN 1 THEN '业务办理助手' WHEN 2 THEN '网点服务地图' WHEN 3 THEN '热门产品' else '热门活动' END) AS typeName," +
                    "DATE_FORMAT(view_date, '%Y-%m') AS dateType,SUM(times) AS totalTimes");
        } else {
            viewsWrapper.setSqlSelect("type,(CASE type WHEN 1 THEN '业务办理助手' WHEN 2 THEN '网点服务地图' WHEN 3 THEN '热门产品' else '热门活动' END) AS typeName," +
                    "DATE_FORMAT(view_date, '%Y-%m-%d') AS dateType,SUM(times) AS totalTimes");
        }

        viewsWrapper.where("1=1");
        if (startDate != null && endDate != null) {
            viewsWrapper.between("view_date", startDate, endDate);
        }
        viewsWrapper.groupBy("type,dateType");
        List<PageViews> viewsList = this.pageViewsService.selectList(viewsWrapper);

        Map<String, Integer> seriesDataMap = new HashMap<>();
        for (PageViews view : viewsList) {
            seriesDataMap.put(view.getTypeName() + view.getDateType(), view.getTotalTimes());
        }

        List<Date> betweenDates = null;
        try {
            betweenDates = DateUtil.getBetweenDates(sdf.parse(startDate), sdf.parse(endDate), statisticsType);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        Map<String, List<Object>> xAxis = new HashMap<>();
        List<Object> xAxisData = new ArrayList<>();
        List<BarChartData> barChartDataList = new ArrayList<>();
        boolean flag = true;
        for (Object typeName : legendData) {
            List<Object> data = new ArrayList<>();
            BarChartData barChartData = new BarChartData();
            barChartData.setName(typeName.toString());
            for (Date betweenDate : betweenDates) {
                String parse = sdf.format(betweenDate);
                if (flag) {
                    xAxisData.add(parse);
                }
                if (seriesDataMap.get(typeName + parse) != null) {
                    data.add(seriesDataMap.get(typeName + parse));
                } else {
                    data.add(0);
                }
            }
            flag = false;
            barChartData.setData(data);
            barChartDataList.add(barChartData);
        }
        xAxis.put("data", xAxisData);
        barChart.setxAxis(xAxis);
        barChart.setSeries(barChartDataList);
        return barChart;

    }
}

