package cn.wisenergy.chnmuseum.party.web.controller;

import cn.wisenergy.chnmuseum.party.common.util.ExcelUtil;
import cn.wisenergy.chnmuseum.party.model.BankBranchInfo;
import cn.wisenergy.chnmuseum.party.model.Employee;
import cn.wisenergy.chnmuseum.party.model.HotProductActivity;
import cn.wisenergy.chnmuseum.party.model.ProductAppointmentRecord;
import cn.wisenergy.chnmuseum.party.service.PublicService;
import cn.wisenergy.chnmuseum.party.service.impl.BankBranchInfoServiceImpl;
import cn.wisenergy.chnmuseum.party.service.impl.EmployeeServiceImpl;
import cn.wisenergy.chnmuseum.party.service.impl.HotProductActivityServiceImpl;
import cn.wisenergy.chnmuseum.party.service.impl.ProductAppointmentRecordServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import cn.wisenergy.chnmuseum.party.web.controller.base.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 * 产品活动预约记录 前端控制器
 * </p>
 *
 * @author 杨智平
 * @since 2018-08-29
 */
@Api(tags = "产品活动预约记录")
@RestController
@RequestMapping("/productAppointmentRecord")
public class ProductAppointmentRecordController extends BaseController {

    private static final Logger logger = LoggerFactory.getLogger(EmployeeController.class);

    @Resource
    private ProductAppointmentRecordServiceImpl productAppointmentRecordService;

    @Resource
    private BankBranchInfoServiceImpl bankBranchInfoService;

    @Resource
    private HotProductActivityServiceImpl hotProductActivityService;

    @Resource
    private EmployeeServiceImpl employeeService;

    @Resource
    private PublicService publicService;

    /**
     * 获取单个产品活动预约记录
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "获取单个产品活动预约记录")
    @GetMapping(value = "/getById")
    @RequiresPermissions("/productAppointmentRecord/getById")
    public ResponseEntity<ProductAppointmentRecord> getById(String id) {
        try {
            ProductAppointmentRecord one = this.productAppointmentRecordService.selectById(id);
            BankBranchInfo bankBranch = this.bankBranchInfoService.selectById(one.getBankBranchId());
            HotProductActivity hotProductActivity = this.hotProductActivityService.selectById(one.getProActId());
            one.setBankBranchInfo(bankBranch);
            one.setProActName(hotProductActivity.getName());
            if (null != one) {
                return ResponseEntity.ok(one);
            }
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(null);
        } catch (Exception e) {
            logger.error("获取单个产品活动预约记录出错!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * 获取产品活动预约记录列表
     *
     * @param proActName
     * @param id
     * @param bankId
     * @param type
     * @return
     */
    @ApiOperation(value = "获取产品活动预约记录列表")
    @GetMapping(value = "/getList")
    @RequiresPermissions("/productAppointmentRecord/getList")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "proActName", value = "活动产品名称", required = false, dataType = "String"),
            @ApiImplicitParam(name = "id", value = "预约记录Id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "bankId", value = "网点Id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "1-产品2-活动", required = true, dataType = "String"),
            @ApiImplicitParam(name = "bookPeopleId", value = "预约人ID", required = false, dataType = "String")})
    public ResponseEntity<Page<ProductAppointmentRecord>> getList(String proActName, String id,
                                                                  String bankId, String type, String bookPeopleId,
                                                                  String roleId, String currentBankId) {
        try {
            if (!"3".equals(roleId)) {
                roleId = "0";
            }
            Map<String, Object> map = new HashMap<>();
            map.put("proActName", proActName);
            map.put("id", id);
            map.put("bankId", bankId);
            map.put("type", type);
            map.put("bookPeopleId", bookPeopleId);
            map.put("roleId", roleId);
            map.put("currentBankId", currentBankId);
            Page<ProductAppointmentRecord> page = getPage();
            Page<ProductAppointmentRecord> pPage = this.productAppointmentRecordService.getPARList(page, map);
            return ResponseEntity.ok(pPage);
        } catch (Exception e) {
            logger.error("服务器错误!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * App提交预约信息
     */
    @Transactional
    @ApiOperation("App提交预约信息")
    @PostMapping(value = "/add")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "proActId", value = "预约产品或者活动的ID", required = true, dataType = "String"),
            @ApiImplicitParam(name = "name", value = "预约人姓名", required = false, dataType = "String"),
            @ApiImplicitParam(name = "phoneNumber", value = "预约人手机号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "detail", value = "预约详情", dataType = "String"),
            @ApiImplicitParam(name = "bankBranchId", value = "预约所属支行", required = true, dataType = "String"),
            @ApiImplicitParam(name = "bookPeopleId", value = "预约人ID", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "1-产品2-活动", required = true)})
    public ResponseEntity<Map<String, Object>> add(ProductAppointmentRecord entity) {
        Map<String, Object> resultMap = new LinkedHashMap<String, Object>();
        try {
            boolean ret = false;

            HotProductActivity hotProductActivity = this.hotProductActivityService.selectById(entity.getProActId());
            if (hotProductActivity == null || hotProductActivity.getIsShow() == 0
                    || hotProductActivity.getIsShow() == 1 || hotProductActivity.getIsShow() == 2
                    || hotProductActivity.getIsShow() == 5) {
                resultMap.put("status", 400);
                if (entity.getType() == 1) {
                    resultMap.put("message", "您预约的产品不存在或已下线！");
                } else {
                    resultMap.put("message", "您预约的活动不存在或已下线！");
                }
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
            } else if (hotProductActivity != null && hotProductActivity.getType() == 1
                    && hotProductActivity.getMaxMoney() < entity.getMoney()) {
                //限制产品最大预约金额
                resultMap.put("status", 400);
                resultMap.put("message", "您的预约金额不能超过" + hotProductActivity.getMaxMoney() + "元！");
            } else if (hotProductActivity != null && hotProductActivity.getType() == 2
                    && hotProductActivity.getMaxNumber() < entity.getNumberOfPeople()) {
                //限制活动最大预约人数
                resultMap.put("status", 400);
                resultMap.put("message", "您的预约人数不能超过" + hotProductActivity.getMaxNumber() + "人！");
            }

            //接收人员工号，多人使用|分割，如"01014760|01158451"
            String recUsrNbr = "";
            if (entity.getBankBranchId() != null) {
                Employee emp = this.employeeService.getEmpCodeBybankId(entity.getBankBranchId());
                if (emp != null) {
                    recUsrNbr = emp.getCode();
                    entity.setRecipientId(emp.getId());
                }
            }
            entity.setName(StringUtils.trimToNull(entity.getName()));
            entity.setPhoneNumber(StringUtils.trimToNull(entity.getPhoneNumber()));
            entity.setCreateTime(new Date(System.currentTimeMillis()));
            entity.setUpdateTime(entity.getCreateTime());
            ret = this.productAppointmentRecordService.insert(entity);
            if (!ret) {
                resultMap.put("status", 400);
                resultMap.put("message", "提交失败");
                return ResponseEntity.status(HttpStatus.BAD_REQUEST).body(resultMap);
            }

            if (!"".equals(recUsrNbr)) {
                //招乎正文，要求以[发送人姓名OR发送人部室]正文格式
                String zhCnt = "";
                if (entity.getType() == 1) {
                    //产品
                    zhCnt = "客户" + entity.getName() + "（" + entity.getPhoneNumber()
                            + "）预约了产品：" + entity.getProActName() + "，预约金额：" +
                            entity.getMoney() + "，请及时联系客户！";
                } else if (entity.getType() == 2) {
                    //活动
                    zhCnt = "客户" + entity.getName() + "（" + entity.getPhoneNumber()
                            + "）预约了活动：" + entity.getProActName() + "，预约人数：" +
                            entity.getNumberOfPeople() + "，请及时联系客户！";
                }
                logger.info(zhCnt);
                //发送消息给大堂主管
                this.publicService.sendMessage(zhCnt, recUsrNbr);
            }

            // 201
            resultMap.put("status", 201);
            resultMap.put("message", "提交成功");
            return ResponseEntity.status(HttpStatus.CREATED).body(resultMap);
        } catch (Exception e) {
            logger.error("提交错误!", e);
        }
        resultMap.put("status", 500);
        resultMap.put("message", "操作失败");
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(resultMap);
    }

    /**
     * App获取产品活动预约记录列表
     *
     * @param bankId
     * @param type
     * @param bookPeopleId
     * @return
     */
    @ApiOperation(value = "App获取产品活动预约记录列表")
    @GetMapping(value = "/getListOnApp")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "bankId", value = "网点Id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "type", value = "1-产品2-活动", required = true, dataType = "String"),
            @ApiImplicitParam(name = "bookPeopleId", value = "预约人ID", required = true, dataType = "String")})
    public ResponseEntity<List<ProductAppointmentRecord>> getListOnApp(String bankId, String type, String bookPeopleId) {
        try {
            Map<String, Object> map = new HashMap<>();
            map.put("bankId", bankId);
            map.put("type", type);
            map.put("bookPeopleId", bookPeopleId);
            List<ProductAppointmentRecord> pList = this.productAppointmentRecordService.getPARListOnApp(map);
            return ResponseEntity.ok(pList);
        } catch (Exception e) {
            logger.error("服务器错误!", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

    /**
     * 导出excel表
     *
     * @param proActName
     * @param id
     * @param bankId
     * @param type
     * @param bookPeopleId
     * @return
     */
    @ApiOperation(value = "导出excel表", notes = "导出excel表")
    @GetMapping(value = "/generalCaseDownload")
    public ResponseEntity<String> proAppoRecordDownload(HttpServletRequest request,
                                                        String proActName, String id,
                                                        String bankId, String type, String bookPeopleId,
                                                        String roleId, String currentBankId) {
        if (!"3".equals(roleId)) {
            roleId = "0";
        }
        /*文件名*/
        String fileName = "热门产品预约记录" + ".xls";
        /*sheet名*/
        String sheetName = "热门产品预约记录" + ".xls";
        /*标题*/
        String[] title = new String[]{"预约人", "预约金额(元)", "预约手机号", "产品名称", "预约时间", "归属网点"};
        if (Integer.parseInt(type) == 2) {
            fileName = "热门活动预约记录" + ".xls";
            sheetName = "热门活动预约记录" + ".xls";
            title = new String[]{"预约人", "预约人数", "预约手机号", "活动名称", "预约时间", "归属网点"};
        }

        Map<String, Object> map = new HashMap<>();
        map.put("proActName", proActName);
        map.put("id", id);
        map.put("bankId", bankId);
        map.put("type", type);
        map.put("bookPeopleId", bookPeopleId);
        map.put("roleId", roleId);
        map.put("currentBankId", currentBankId);
        List<ProductAppointmentRecord> pList = this.productAppointmentRecordService.downLoadPAR(map);

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String[][] values = new String[pList.size()][];
        DecimalFormat df = new DecimalFormat("#.##");
        for (int i = 0; i < pList.size(); i++) {
            values[i] = new String[title.length];
            //将对象内容转换成string
            ProductAppointmentRecord obj = pList.get(i);
            values[i][0] = obj.getName();
            if (Integer.parseInt(type) == 1) {
                values[i][1] = df.format(obj.getMoney());
            } else {
                values[i][1] = String.valueOf(obj.getNumberOfPeople());
            }
            values[i][2] = obj.getPhoneNumber();
            values[i][3] = obj.getProActName();
            values[i][4] = sdf.format(obj.getCreateTime());
            values[i][5] = obj.getBankName();
        }

        HSSFWorkbook wb = ExcelUtil.getHSSFWorkbook(sheetName, title, values, null);

        //将文件存到指定位置
        try {
            //对文件名进行编码
            fileName = ExcelUtil.encodeFileName(fileName, request);
            ExcelUtil.setResponseHeader(response, fileName);
            OutputStream os = response.getOutputStream();
            wb.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            logger.error("下载失败");
        }
        return ResponseEntity.ok("下载成功");
    }

}

