package cn.wisenergy.chnmuseum.party.web.controller;

import cmb.netpayment.Security;
import cn.wisenergy.chnmuseum.party.web.controller.base.BaseController;
import com.sun.xml.fastinfoset.Encoder;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import sun.misc.BASE64Decoder;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.security.SecureRandom;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * <p>
 * 公共方法接口
 * </p>
 *
 * @author 杨建龙
 * @since 2018-10-11
 */
@Api(tags = "公共方法接口")
@RestController
@RequestMapping("/public")
public class PublicController extends BaseController {

    private static final Logger logger = LoggerFactory.getLogger(PublicController.class);

    @Value("${CMB_HOME_URL}")
    private String CMB_HOME_URL;

    @Value("${CMB_CORPKEY}")
    private String CMB_CORPKEY;

    /**
     * 返回首页方法
     */
    @GetMapping(value = "/backHomepage")
    public void backHomepage(HttpServletResponse response, String userId, String bankId, String timeStamp) throws Exception {
        logger.info(CMB_HOME_URL + "?userId=" + userId + "&bankId=" + bankId);
        response.sendRedirect(CMB_HOME_URL + "?userId=" + userId + "&bankId=" + bankId+"&showtopbar=false");
    }

    @PostMapping(value = "/getcmbxml")
    @ResponseBody
    public void getcmbxml(String param, String sResponseXml) throws Exception {

        //用dom4j将暗文xml转成dom树
        Document document = DocumentHelper.parseText(sResponseXml);
        // 获取根节点
        Element rootElement = document.getRootElement();
        // 获取根节点下的子节点head
        Iterator iter = rootElement.elementIterator("Head");
        String ResultType = "";
        String CryptType = "";
        // 遍历head节点
        while (iter.hasNext()) {
            Element recordEle = (Element) iter.next();
            // 拿到head节点下的子节点ResultType值
            ResultType = recordEle.elementTextTrim("ResultType");
            // 拿到head节点下的子节点CryptType值
            CryptType = recordEle.elementTextTrim("CryptType");
        }
        // 拿到根节点下的子节点Body值
        String UserDataBase64 = rootElement.elementTextTrim("Body");

        if (ResultType.equals("Y")) {
            //登录成功进行转码、解密、验证签名操作
            String loginInfo = UserDataBase64;
            if ("2".equals(CryptType)) {
                //进行解密数据
                BASE64Decoder base64decoder = new BASE64Decoder();
                loginInfo = new String(decrypt(base64decoder.decodeBuffer(UserDataBase64), CMB_CORPKEY));
            }
            String stringValue = loginInfo.substring(loginInfo.indexOf("<Body>") + 6, loginInfo.indexOf("</Body>"));
            //用dom4j将登录信息明文xml转成dom树
            Document loginInfoDom = DocumentHelper.parseText(loginInfo);
            // 获取根节点
            Element loginInfoDomRoot = loginInfoDom.getRootElement();
            // 获取根节点下的子节点Body
            Iterator bodyIter = loginInfoDomRoot.elementIterator("Body");
            //用户ID，由身份证+姓名生成
            String uniqueUserID = "";
            // 拿到Body节点下的子节点TimeStamp值
            String timeStamp = "";
            while (bodyIter.hasNext()) {
                Element bodyEle = (Element) bodyIter.next();
                /*// 拿到Body节点下的子节点CorpNo值
                String CorpNo = bodyEle.elementTextTrim("CorpNo");
                // 拿到Body节点下的子节点CorpName值
                String CorpName = bodyEle.elementTextTrim("CorpName");
                // 拿到Body节点下的子节点AppName值
                String AppName = bodyEle.elementTextTrim("AppName");
                // 拿到Body节点下的子节点AppVersion值
                String AppVersion = bodyEle.elementTextTrim("AppVersion");
                // 拿到Body节点下的子节点UserID值
                String userID = bodyEle.elementTextTrim("UserID");
                // 拿到Body节点下的子节点NewUserID值
                String NewUserID = bodyEle.elementTextTrim("NewUserID");*/
                // 拿到Body节点下的子节点UniqueUserID值
                uniqueUserID = bodyEle.elementTextTrim("UniqueUserID");
                // 拿到Body节点下的子节点TimeStamp值
                timeStamp = bodyEle.elementTextTrim("TimeStamp");
            }
            // 获取根节点下的子节点Tail
            Iterator tailIter = loginInfoDomRoot.elementIterator("Tail");
            String verify = "";
            while (tailIter.hasNext()) {
                Element tailEle = (Element) tailIter.next();
                // 拿到Tail节点下的子节点Verify值
                verify = tailEle.elementTextTrim("Verify");
            }

            //获取public.key路径
            Resource resource = new ClassPathResource("public.key");
            File file = resource.getFile();
            String canonicalPath = file.getCanonicalPath();
            //验证签名
            boolean flag = this.verifySignature(stringValue + "&signature=" + verify, canonicalPath);
            if (flag) {
                //验证成功之后返回首页
                this.backHomepage(response, uniqueUserID, param, timeStamp);
            }
        } else if (ResultType.equals("N")) {
            //登录失败返回消息
            throw new Exception("登录失败！失败原因：" + UserDataBase64);
        }
    }

    /**
     * 解密
     *
     * @param src byte[]
     * @param key String
     * @return byte[]
     * @throws Exception
     */
    private static byte[] decrypt(byte[] src, String key) throws Exception {
        // DES算法要求有一个可信任的随机数源
        SecureRandom random = new SecureRandom();
        // 创建一个DESKeySpec对象
        DESKeySpec desKey = new DESKeySpec(key.getBytes("UTF-8"));
        // 创建一个密匙工厂
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
        // 将DESKeySpec对象转换成SecretKey对象
        SecretKey securekey = keyFactory.generateSecret(desKey);
        // Cipher对象实际完成解密操作
        Cipher cipher = Cipher.getInstance("DES");
        // 用密匙初始化Cipher对象
        cipher.init(Cipher.DECRYPT_MODE, securekey, random);
        // 真正开始解密操作
        return cipher.doFinal(src);
    }

    /**
     * 验证签名
     */
    public static boolean verifySignature(String signdata, String publickeypath) throws Exception {
        Security verifier = new Security(publickeypath);
        byte[] signdataByte = signdata.getBytes(Encoder.UTF_8);
        return verifier.checkInfoFromBank(signdataByte);//成功时返回true
    }

    /**
     * 验证token
     *
     * @return
     */
    @ApiOperation(value = "验证token")
    @GetMapping(value = "/authToken")
    @RequiresPermissions("/public/authToken")
    public ResponseEntity<Map<Object, String>> authToken() {
        try {
            Map<Object, String> map = new HashMap<>();
            map.put("status", "200");
            map.put("message", "验证成功");
            return ResponseEntity.status(HttpStatus.OK).body(map);
        } catch (Exception e) {
            logger.error("服务器错误！", e);
        }
        return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(null);
    }

}

