package cn.wisenergy.chnmuseum.party.web.controller.base;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import cn.wisenergy.chnmuseum.party.auth.util.JwtTokenUtil;
import cn.wisenergy.chnmuseum.party.common.vo.AjaxResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

public class SuperController {

	protected Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	protected HttpServletRequest request;

	@Autowired
	protected HttpServletResponse response;

	@Autowired
	protected HttpSession session;

	@Autowired
	protected ServletContext application;

	/**
	 * <p>
	 * 获取分页对象
	 * </p>
	 */
	protected <T> Page<T> getPage() {
		return getPage(10);
	}


	/**
	 * <p>
	 * 根据token信息
	 * </p>
	 */
	protected String getUserId() {

		String authorization = request.getHeader("Authorization");
		if (StringUtils.isNotBlank(authorization)) {
			if (authorization.startsWith("Bearer ")) {
				authorization = authorization.substring(7);
			}
		}
		String employeeId= JwtTokenUtil.getEmployeeId(authorization);
		return employeeId;
	}

	/**
	 * <p>
	 * 根据token信息
	 * </p>
	 */
	protected String getUserName() {

		String authorization = request.getHeader("Authorization");
		if (StringUtils.isNotBlank(authorization)) {
			if (authorization.startsWith("Bearer ")) {
				authorization = authorization.substring(7);
			}
		}
		String username=JwtTokenUtil.getUsername(authorization);
		return username;
	}

	/**
	 * <p>
	 * 获取分页对象
	 * </p>
	 *
	 * @param size
	 *            每页显示数量
	 * @return
	 */
	protected <T> Page<T> getPage(int size) {
		int _size = size, _index = 1;
		// 排序方式 asc升序  desc降序
		boolean isAsc = false;
		// 排序的字段
		String orderByField = null;
		if (request.getParameter("_size") != null) {
			_size = Integer.parseInt(request.getParameter("_size"));
		}
		if (request.getParameter("_index") != null) {
			_index = Integer.parseInt(request.getParameter("_index"));
		}
		if (request.getParameter("_sortBy") != null) {
			orderByField = request.getParameter("_sortBy");
		}
		if (request.getParameter("_isAsc") != null) {
			isAsc = Boolean.valueOf(request.getParameter("_isAsc"));
		}
		if (orderByField == null) {
			return new Page<>(_index, _size);
		}
		return new Page<>(_index, _size, orderByField, isAsc);
	}

	/**
	 * 重定向至地址 url
	 *
	 * @param url
	 *            请求地址
	 * @return
	 */
	protected String redirectTo(String url) {
		StringBuffer rto = new StringBuffer("redirect:");
		rto.append(url);
		return rto.toString();
	}

	/**
	 *
	 * 返回 JSON 格式对象
	 *
	 * @param object
	 *            转换对象
	 * @return
	 */
	protected String toJson(Object object) {
		return JSON.toJSONString(object, SerializerFeature.BrowserCompatible);
	}

	/**
	 *
	 * 返回 JSON 格式对象
	 *
	 * @param object
	 *            转换对象
	 * @return
	 */
	protected String toJson(Object object, String format) {
		if (format == null) {
			return toJson(object);
		}
		return JSON.toJSONStringWithDateFormat(object, format, SerializerFeature.WriteDateUseDateFormat);
	}

	/**
	 * <p>
	 * 自动判定是否有跨域操作,转成字符串并返回
	 * </p>
	 *
	 * @param object
	 * @return 跨域或不跨域的字符串
	 */
	protected String callback(AjaxResult object) {
		return callback(object, null);
	}

	protected String callback(AjaxResult object, String format) {
		String callback = request.getParameter("callback");
		if (callback == null) {
			/**
			 * 非 JSONP 请求
			 */
			return toJson(object, format);
		}
		StringBuilder json = new StringBuilder();
		json.append(callback);
		json.append("(").append(toJson(object, format)).append(")");
		return json.toString();
	}

	protected String callbackSuccess(Object obj) {
		return callback(new AjaxResult(obj));
	}

	protected String callbackFail(String message) {
		return callback(new AjaxResult(false, message));
	}

}
